------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  MMI.vhd                                         --
--  Version 2012-06-13-01                           --
--                                                  --
------------------------------------------------------


-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity MMI is
	port
	(
		-- inputs
		clk_50MHz 		: in std_logic;		-- 50 MHz Takt
		reset				: in std_logic; 		-- Rücksetzeingang
		next_filter 	: in std_logic;		-- nächsten Filter auswählen
		prior_filter 	: in std_logic;		-- vorherigen Filter auswählen
		inc 				: in std_logic;		-- Verstärkungsfaktor Filter/Volume inkrementieren (1dB)
		dec 				: in std_logic;		-- Verstärkungsfaktor Filter/Volume dekrementieren (-1dB)
		seg_off			: in std_logic;		-- 7-Segement-Anzeigen ein-/ausschalten (0 -> ein, 1 -> aus);
		jd					: in std_logic;
		
		-- outputs
		volume 		: out signed(7 downto 0);		-- Verstärkungsfaktor Lautstärke in dB
		filter1 		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter1 in dB
		filter2 		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter2 in dB
		filter3		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter3 in dB
		filter4		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter4 in dB
		filter5		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter5 in dB
		filter6		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter6 in dB
		filter7		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter7 in dB
		filter8		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter8 in dB
		filter9		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter9 in dB
		filter10 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter10 in dB
		filter11 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter11 in dB
		filter12 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter12 in dB
		filter13 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter13 in dB
		filter14 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter14 in dB
		LCD_EN 		: out std_logic;					-- LCD Ein-/Auschaltsignal
		LCD_RW 		: out std_logic;					-- Schreib-/Lesesignal an LCD
		LCD_RS 		: out std_logic;					-- RS LCD
		LCD_ON 		: out std_logic;					-- Ein-/Ausschaltsignal LCD komplett
		LCD_BLON 	: out std_logic;					-- Ein-/Ausschaltsignal Hintergrundbeleuchtung LCD
		filter_ones : out unsigned(6 downto 0);	-- 7-Segment-Anzeige Auswahlfilter Einer-Stelle
		filter_ten 	: out unsigned(6 downto 0);	-- 7-Segment-Anzeige Auswahlfilter Zehner-Stelle
		db_ones 		: out unsigned(6 downto 0);	-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Einer-Stelle
		db_ten 		: out unsigned(6 downto 0);	-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Zehner-Stelle
		db_sign 		: out unsigned(6 downto 0);	-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Vorzeichen
		db_letter_1 : out unsigned(6 downto 0);	-- 7-Segement-Anzeige Buchstebe 'd' für db
		db_letter_2 : out unsigned(6 downto 0);	-- 7-Segement-Anzeige Buchstebe 'b' für db
	
		-- inouts
		LCD_DATA : inout unsigned(7 downto 0)	-- LCD - Daten BUS
	);
end MMI;



architecture behavior of MMI is

	signal lcd1 :  unsigned(11 downto 0) := x"120";	-- Zeichen 1 zur Anzeige des ausgewähltn Filters
	signal lcd2 :  unsigned(11 downto 0) := x"120";	-- Zeichen 2 zur Anzeige des ausgewähltn Filters
	signal lcd3 :  unsigned(11 downto 0) := x"120";	-- Zeichen 3 zur Anzeige des ausgewähltn Filters
	signal lcd4 :  unsigned(11 downto 0) := x"120"; -- Vorzeichen des Verstärkungsfaktors
	signal lcd5 :  unsigned(11 downto 0) := x"120"; -- Zehner-Stelle des Verstärkungsfaktors in dB
	signal lcd6 :  unsigned(11 downto 0) := x"120"; -- Einer-Stelle des Verstärkungsfktors in dB

	-- MMI Controller einbinden (Verarbeitung Tasten-/Schaltereingaben)
	component mmi_controller
		port
		(
			-- inputs
			clk_50MHz	 	: in std_logic;		-- 50 MHz Takt
			next_filter	 	: in std_logic;		-- nächsten Fitler auswählen
			prior_filter 	: in std_logic;		-- vorherigen Filter ausählen
			inc				: in std_logic; 		-- Verstärkungsfaktor Filter/Volume inkrementieren (1dB)
			dec				: in std_logic;		-- Verstärkungsfaktor Filter/Volume inkrementieren (1dB)
			seg_off			: in std_logic;		-- 7-Segement-Anzeigen ein-/ausschalten (0 -> ein, 1 -> aus);
			reset				: in std_logic;		-- Rücksetzeingang
			jd					: in std_logic;

			-- outputs
			lcd_1			: out unsigned(11 downto 0) := x"120";			-- Zeichen 1 zur Anzeige des ausgewähltn Filters
			lcd_2			: out unsigned(11 downto 0) := x"120";			-- Zeichen 2 zur Anzeige des ausgewähltn Filters
			lcd_3			: out unsigned(11 downto 0) := x"120";			-- Zeichen 3 zur Anzeige des ausgewähltn Filters
			lcd_4			: out unsigned(11 downto 0) := x"120";			-- Vorzeichen des Verstärkungsfaktors
			lcd_5			: out unsigned(11 downto 0) := x"120";			-- Zehner-Stelle des Verstärkungsfaktors in dB
			lcd_6			: out unsigned(11 downto 0) := x"120";			-- Einer-Stelle des Verstärkungsfaktors in dB
			vol_value	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Lautstärke in dB
			f1_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 1 in dB
			f2_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 2 in dB
			f3_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 3 in dB
			f4_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 4 in dB
			f5_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 5 in dB
			f6_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 6 in dB
			f7_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 7 in dB
			f8_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 8 in dB
			f9_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 9 in dB
			f10_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 10 in dB
			f11_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 11 in dB
			f12_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 12 in dB
			f13_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 13 in dB
			f14_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 14 in dB
			filter_ones	: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Auswahlfilter Einer-Stelle
			filter_ten	: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Auswahlfilter Zehner-Stelle
			db_ones		: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Einer-Stelle
			db_ten		: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Zehner-Stelle
			db_sign		: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Vorzeichen
			db_letter_1 : out unsigned(6 downto 0);						-- 7-Segement-Anzeige Buchstebe 'd' für db
			db_letter_2 : out unsigned(6 downto 0)							-- 7-Segement-Anzeige Buchstebe 'b' für db
		);
	end component;

	component lcd_controller
		port
		(
			-- inputs
			clk_50MHz	: in std_logic;						-- 50 MHz Takt
			reset			: in std_logic;						-- Rücksetzeingang
			lcd_1			: in unsigned(11 downto 0);		-- Zeichen 1 zur Anzeige des ausgewühltn Filters
			lcd_2			: in unsigned(11 downto 0);		-- Zeichen 2 zur Anzeige des ausgewühltn Filters
			lcd_3			: in unsigned(11 downto 0);		-- Zeichen 3 zur Anzeige des ausgewühltn Filters
			lcd_4			: in unsigned(11 downto 0);		-- Vorzeichen des Verstärkungsfaktors
			lcd_5			: in unsigned(11 downto 0);		-- Zehner-Stelle des Verstärkungsfaktors in dB
			lcd_6			: in unsigned(11 downto 0);		-- Einer-Stelle des Verstärkungsfaktors in dB

			-- outputs
			lcd_en		: out std_logic;	-- LCD Ein-/Auschaltsignal
			lcd_rw		: out std_logic;	-- Schreib-/Lesesignal an LCD
			lcd_rs		: out std_logic;	-- RS LCD
			lcd_on		: out std_logic;	-- Ein-/Ausschaltsignal LCD komplett
			lcd_blon	: out std_logic;		-- Ein-/Ausschaltsignal Hintergrundbeleuchtung LCD
			
			-- inouts
			lcd_data 	: inout unsigned(7 downto 0)	-- LCD - Daten BUS
		);
	end component;

begin
	
	-- LCD Controller Ports zuweisen
	lcd_controller_1: lcd_controller port map 
	(
		clk_50MHz,
		reset,
		lcd1,
		lcd2,
		lcd3,
		lcd4,
		lcd5,
		lcd6,
		LCD_EN,
		LCD_RW,
		LCD_RS,
		LCD_ON,
		LCD_BLON,
		LCD_DATA
	);


	-- MMI Controller Ports zuweisen
	mmi_controller_1: mmi_controller port map
	(
		clk_50MHz,
		next_filter,
		prior_filter,
		inc,
		dec,
		seg_off,
		reset,
		jd,
		lcd1,
		lcd2,
		lcd3,
		lcd4,
		lcd5,
		lcd6,
		volume,
		filter1,
		filter2,
		filter3,
		filter4,
		filter5,
		filter6,
		filter7,
		filter8,
		filter9,
		filter10,
		filter11,
		filter12,
		filter13,
		filter14,
		filter_ones,
		filter_ten,
		db_ones,
		db_ten,
		db_sign,
		db_letter_1,
		db_letter_2
	);

end behavior;
