------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  audio_input_register.vhd                        --
--  Version 2012-06-09-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity audio_input_register is
    port 
	(
		-- inputs
		adclrc	: in std_logic;	-- 0 -> right channel; 1 -> left channel
		adcdat	: in std_logic;	-- serial audio data
		bclk		: in std_logic;	-- bit clock (Übernahme der einzelnen Bits der Audiodaten bei steigender Flanke)
	
		-- outputs
		dl		: out signed(23 downto 0);		-- parallel audio output left channel
		dr		: out signed(23 downto 0); 	-- parallel audio output right channel
		enl	: out std_logic;					-- read audio data left channel enable
		enr	: out std_logic					-- read audio data right channel enable
    );
end audio_input_register;



architecture behavior of audio_input_register is
begin

	process
	
		variable data  			: signed(23 downto 0);		-- Zwischenspeicher für Audio-Eingangsdaten
		variable counter_left 	: unsigned(4 downto 0);		-- Zähler für workflow zur Übernahme der Audiodaten linker Kanal
		variable counter_right 	: unsigned(4 downto 0);		-- Zähler für workflow zur Übernahme der Audiodaten rechter Kanal
		
	begin
		
		-- Warten auf steigende Flanke des Bit Clock (BCLK) des Audio-Codecs
		wait until rising_edge(bclk);
		
		-- Übernahme der seriellen Daten des linken Kanals vom ADC und
		-- Wandlung in ein paralleles Datenwort
		if (adclrc='1') then
			if (counter_left < "11000") then data := data(22 downto 0) & adcdat; end if;
			case counter_left is
				when "11000" => enl <= '0';
				when "11001" => dl <= data;
				when "11010" => enl <= '1';
				when "11011" => data := (others => '0');
				when others => null;
			end case;
			if (counter_left < "11100") then counter_left := counter_left + 1; end if;
			counter_right := (others => '0');
		else
			-- Übernahme der seriellen Daten des rechten Kanals vom ADC und
			-- Wandlung in ein paralleles Datenwort
			if (counter_right < "11000") then data := data(22 downto 0) & adcdat; end if;
			case counter_right is
				when "11000" => enr <= '0';
				when "11001" => dr <= data;
				when "11010" => enr <= '1';
				when "11011" => data := (others => '0');
				when others => null;
			end case;
			if (counter_right < "11100") then counter_right := counter_right + 1; end if;
			counter_left := (others => '0');
		end if;
		
	end process;
	
end behavior;
