------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  audio_output_register.vhd                       --
--  Version 2012-06-09-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity audio_output_register is
    port 
	(
		-- inputs
		dl			: in signed(23 downto 0);	-- parallel audio data left channel
		dr			: in signed(23 downto 0);	-- parallel audio data right channel
		enl		: in std_logic;				-- read left channel enable
		enr		: in std_logic;				-- read right channel enable
		bclk		: in std_logic;				-- bit clock (Übergabe der einzelnen Bits der Audiodaten bei fallender Flanke)
		daclrc	: in std_logic;				-- 0 -> right channel; 1 -> left channel
		
		-- outputs
		dacdat 	: out std_logic				-- serial audio data output
    );

end audio_output_register;



architecture behavior of audio_output_register is
	
	signal idl : signed(23 downto 0);	-- intern parallel audio data left channel 
	signal idr : signed(23 downto 0); 	-- intern parallel audio data right channel

	begin
	
		-- Nach steigende Flanke des Übernahmesignals linker Kanal
		-- Daten übernehmen und im Signal idl speichern
		read_left_channel: process begin
			wait until rising_edge(enl);
			idl <= dl;
		end process read_left_channel;

		-- Nach steigende Flanke des Übernahmesignals rechter Kanal
		-- Daten übernehmen und im Signal idr speichern
		read_right_channel: process begin
			wait until rising_edge(enr);
			idr <= dr;
		end process read_right_channel;
	
		-- Der folgende Process sendet die in parallelr Struktur
		-- vorliegenden Daten seriell an den DAC
		transmit: process 
		
			variable sdl 				: signed(23 downto 0);	-- zu sendende Audiodaten linker Kanal
			variable sdr 				: signed(23 downto 0);	-- zu sendende Audiodaten rechter Kanal
			variable counter_left 	: unsigned(4 downto 0);	-- Zähler für workflow Senden linker Kanal
			variable counter_right 	: unsigned(4 downto 0); -- Zähler für workflow Senden rechter Kanal
			
		begin
			
			-- Warten auf fallende Flanke bit clock
			-- (Übergabe der einzelnen Bits der Audiodaten bei fallender Flanke)
			wait until falling_edge(bclk);
			
			if (daclrc='1') then		
				-- Senden der Audiodaten linker Kanal
				if (counter_left < "11000") then 
					counter_left := counter_left + 1; 
					dacdat <= sdl(23);
					sdl := sdl(22 downto 0) & '0';
				else
					counter_right := "00000";
					sdr := idr;
					dacdat <= sdr(23);
					sdr := sdr(22 downto 0) & '0';
				end if;
			else			
				-- Senden der Audiodaten rechter Kanal
				if (counter_right <= "11000") then 
					counter_right := counter_right + 1; 
					dacdat <= sdr(23);
					sdr := sdr(22 downto 0) & '0';
				else 
					counter_left := "00000";
					sdl := idl;
					dacdat <= sdl(23);
					sdl := sdl(22 downto 0) & '0';
				end if;
			end if;
			
		end process;
end behavior;
