------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  clock_divider.vhd                               --
--  Version 2012-06-09-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity clock_divider is
	port
	(
		-- inputs
		clk_10MHz 	: in std_logic;				-- 10 MHz Takteingang
	
		-- outputs
		clk_400kHz : out std_logic := '0';		-- 400 kHz Taktausgang
		clk_100Hz  : out std_logic := '0'		-- 100 Hz Taktausgang
	);
end clock_divider;



architecture behavior of clock_divider is
	
	signal c400 : unsigned(8 downto 0) := (others => '0');		-- Zähler für 400 kHz Takt
	signal c100 : unsigned(19 downto 0) := (others => '0');		-- Zähler für 20 Hz Takt

begin

	process begin
		
		-- auf steigende Flanke des 10 MHz Taktes warten
		wait until rising_edge(clk_10MHz);
		
		-- Zähler für 400 kHz auswerten und entsprechend den Taktausgang stellen
		case c400 is
			when "000000101" => clk_400kHz <= '1';
			when "000001001" => clk_400kHz <= '0';
			when others => null;
		end case;

		-- Zähler für 100 Hz auswerten und entsprechend den Taktausgang stellen
		case c100 is
			when x"0C350" => clk_100Hz <= '0';
			when x"1869F" => clk_100Hz <= '1';
			when others => null;
		end case;
		
		-- Zähler inkrementieren und Zählerstand prüfen
		if (c400 = "000001001") then c400 <= (others => '0'); else c400 <= c400 + 1; end if;
		if (c100 = "110010000") then c100 <= (others => '0'); else c100 <= c100 + 1; end if;
		
	end process;

end behavior;
