------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  coefficients_register.vhd                       --
--  Version 2012-10-08-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity coefficients_register is
    port 
	(
		-- inputs
		clk_50MHz	: in std_logic;				-- 50 MHz Takt
		g1  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 1
		g2  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 2				
		g3  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 3				
		g4  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 4				
		g5  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 5				
		g6  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 6				
		g7  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 7				
		g8  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 8				
		g9  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 9				
		g10 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 10				
		g11 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 11				
		g12 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 12				
		g13 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 13				
		g14 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 14				
		
		-- outputs
		b0_1  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  1 des IIR-Filters
		b2_1  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  1 des IIR-Filters
		b0_2  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  2 des IIR-Filters
		b2_2  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  2 des IIR-Filters
		b0_3  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  3 des IIR-Filters
		b2_3  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  3 des IIR-Filters
		b0_4  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  4 des IIR-Filters
		b2_4  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  4 des IIR-Filters
		b0_5  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  5 des IIR-Filters
		b2_5  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  5 des IIR-Filters
		b0_6  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  6 des IIR-Filters
		b2_6  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  6 des IIR-Filters
		b0_7  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  7 des IIR-Filters
		b2_7  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  7 des IIR-Filters
		b0_8  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  8 des IIR-Filters
		b2_8  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  8 des IIR-Filters
		b0_9  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  9 des IIR-Filters
		b2_9  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  9 des IIR-Filters
		b0_10 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  0 des IIR-Filters
		b2_10 		: out signed(63 downto 0);	-- Koeffizient b2 für Filter 10 des IIR-Filters
		b0_11 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter 11 des IIR-Filters
		b2_11 		: out signed(63 downto 0);	-- Koeffizient b2 für Filter 11 des IIR-Filters
		b0_12 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter 12 des IIR-Filters
		b2_12 		: out signed(63 downto 0);	-- Koeffizient b2 für Filter 12 des IIR-Filters
		b0_13 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter 13 des IIR-Filters
		b2_13 		: out signed(63 downto 0);	-- Koeffizient b2 für Filter 13 des IIR-Filters
		b0_14 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter 14 des IIR-Filters
		b2_14 		: out signed(63 downto 0)	-- Koeffizient b2 für Filter 14 des IIR-Filters
    );
end coefficients_register;



architecture behavior of coefficients_register is

	-- Zähler für workflow zur Berechnung der Koeffizienten
	signal counter : unsigned(7 downto 0) := x"00";

	-- Alle weiteren Koeffizienten sind Bewrechnungsgrundlagen für 
	-- die Koeffizienten b0 und b2 der jeweiligen Kaskaden.
	-- Die Koeffizienten werden wie folgt berechnet:
	-- b0 = c1 + v*c2 + c3
	-- b2 = c1 - v*c2 + c3
	-- Dabei ist v der lineare Verstärkungsfaktor der jeweiligen Kaskade
	

	-- Koeffizienten Kaskade 1
	constant c1_1  : signed(63 downto 0) := x"07FED32FF9DE87BF";
	constant c2_1  : signed(63 downto 0) := x"00012BEA2D5C0948";
	constant c3_1  : signed(63 downto 0) := x"000000E5D8C56F08";

	-- Koeffizienten Kaskade 2
	constant c1_2  : signed(63 downto 0) := x"07FE128B3C6E3A7F";
	constant c2_2  : signed(63 downto 0) := x"0001EB0C616922F0";
	constant c3_2  : signed(63 downto 0) := x"000002686228A287";

	-- Koeffizienten Kaskade 3
	constant c1_3  : signed(63 downto 0) := x"07FCD5BE74DAB6BF";
	constant c2_3  : signed(63 downto 0) := x"000323CCF743EE03";
	constant c3_3  : signed(63 downto 0) := x"0000067493E15B2A";

	-- Koeffizienten Kaskade 4
	constant c1_4  : signed(63 downto 0) := x"07FACB76AFF6DDFF";
	constant c2_4  : signed(63 downto 0) := x"0005233C4FBBD171";
	constant c3_4  : signed(63 downto 0) := x"0000114D004D50D5";

	-- Koeffizienten Kaskade 5
	constant c1_5  : signed(63 downto 0) := x"07F76AF7A64E1C7F";
	constant c2_5  : signed(63 downto 0) := x"000866B1E38AD596";
	constant c3_5  : signed(63 downto 0) := x"00002E5676270DAA";

	-- Koeffizienten Kaskade 6
	constant c1_6  : signed(63 downto 0) := x"07F1CAF3B15E2E3F";
	constant c2_6  : signed(63 downto 0) := x"000DB9118BBA16C0";
	constant c3_6  : signed(63 downto 0) := x"00007BFAC2E7BABF";

	-- Koeffizienten Kaskade 7
	constant c1_7  : signed(63 downto 0) := x"07E854D475B6D23F";
	constant c2_7  : signed(63 downto 0) := x"001660078FA1720A";
	constant c3_7  : signed(63 downto 0) := x"00014B23FAA7BB8F";

	-- Koeffizienten Kaskade 8
	constant c1_8  : signed(63 downto 0) := x"07D82FB05E7762BF";
	constant c2_8  : signed(63 downto 0) := x"00245E6A2D32ED67";
	constant c3_8  : signed(63 downto 0) := x"000371E57455AFBF";

	-- Koeffizienten Kaskade 9
	constant c1_9  : signed(63 downto 0) := x"07BC143F0ED0F9FF";
	constant c2_9  : signed(63 downto 0) := x"003ACA8339D1ADFB";
	constant c3_9  : signed(63 downto 0) := x"0009213DB75D5825";

	-- Koeffizienten Kaskade 10
	constant c1_10 : signed(63 downto 0) := x"0789E84064A530BF";
	constant c2_10 : signed(63 downto 0) := x"005E18CB9FA52C2F";
	constant c3_10 : signed(63 downto 0) := x"0017FEF3FBB5A2EA";

	-- Koeffizienten Kaskade 11
	constant c1_11 : signed(63 downto 0) := x"072E0420CC0C187F";
	constant c2_11 : signed(63 downto 0) := x"0093CFE1973E3B6F";
	constant c3_11 : signed(63 downto 0) := x"003E2BFD9CB5AC4F";

	-- Koeffizienten Kaskade 12
	constant c1_12 : signed(63 downto 0) := x"068306CC8552E03F";
	constant c2_12 : signed(63 downto 0) := x"00DFCFCF0707C177";
	constant c3_12 : signed(63 downto 0) := x"009D296473A55E0F";

	-- Koeffizienten Kaskade 13
	constant c1_13 : signed(63 downto 0) := x"0548EB18E10CD2FF";
	constant c2_13 : signed(63 downto 0) := x"0139F9E5E6AF683F";
	constant c3_13 : signed(63 downto 0) := x"017D1B013843C49F";

	-- Koeffizienten Kaskade 14
	constant c1_14 : signed(63 downto 0) := x"033128F55A68E0FF";
	constant c2_14 : signed(63 downto 0) := x"016F4EF78112995F";
	constant c3_14 : signed(63 downto 0) := x"035F88132484857F";

	
	-- Komponente zur Umrechnung eines logarithmischen Verstärkungsfaktors in
	-- einen linearen Verstärkungsfaktors über eine LUT
	component db2linear
		port 
		(
			db 		: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor
			linear 	: out signed(39 downto 0)	-- linearer Verstärkungsfaktor
		);
	end component;
	
	-- Signale zur Übernahme der Umrechnungsergebnisse der logarithmischen Verstärkungsfaktoren 
	-- in die linearen Verstärkungsfaktoren der einzelnen Kaskaden des IIR-Filters
	signal g  : signed(7 downto 0)  := x"00";
	signal v  : signed(39 downto 0) := x"0800000000";
	
	-- Multiplizierer zur Berechnung der Koeffizienten b0 und b2.
	component coeff_mul
		port
		(
			dataa		: in  signed (63 DOWNTO 0);
			datab		: in  signed (39 DOWNTO 0);
			result	: out signed (103 DOWNTO 0)
		);
	end component;
	
	-- Signale zur Übergabe der Operatoren und zur Übernahme des Multiplkationsergebnisses
	signal mul_a : signed(63 downto 0);
	signal mul_b : signed(39 downto 0);
	signal mul_r : signed(103 downto 0);
	
begin
		
	-- LUT db zu linear Komponente Ports zuweisen
	db2linear_1  : db2linear port map (g, v);
	
	-- Multiplizierer Ports zuweisen
	coeff_mul_1 : coeff_mul port map (mul_a, mul_b, mul_r);
	
	-- Prozess zur Berechnung der Koeffizienten b0 und b2 
	-- der einzelnen Kaskaden des IIR-Filters
	process begin
		
		-- Warten auf steigende Flanke des 50 MHz Taktes
		wait until rising_edge(clk_50MHz);
		
		case counter is
		
			-- Bei der Berechnung der Koeffizienten b0 und b2 der 
			-- einzelnen Kaskaden wird wie folgt vorgegangen.
			-- 1. Umrechnung logarithmischer Verstärkungsfaktor in linearen Verstärkungsfaktor  v = 10^(g/20)
			-- 2. Multiplikation des Koeffizienten c2 mit dem linearen Verstärkungsfaktor c2' = c2 * v
			-- 3. Addition bzw. Subtraktion der jeweiligen Koeffizienten zur Berechnung von b0 und b2
			--    b0 = c1 + c2' + c3
			--    b2 = c1 - c2' + c3
			-- 4. Übergabe des Berechnungsergebnisses der Koeffizienten b0 und b2 an den jeweiligen Ausgang

			-- Koeffizeinten b0 und b2 der Kaskade 1 berechnen
			when x"00" => g <= g1;
			when x"02" => mul_a <= c2_1; mul_b <= v;
			when x"05" => b0_1 <= c1_1 + mul_r(98 downto 35) + c3_1; b2_1 <= c1_1 - mul_r(98 downto 35) + c3_1;

			-- Koeffizeinten b0 und b2 der Kaskade 2 berechnen
			when x"0A" => g <= g2;
			when x"0C" => mul_a <= c2_2; mul_b <= v;
			when x"0F" => b0_2 <= c1_2 + mul_r(98 downto 35) + c3_2; b2_2 <= c1_2 - mul_r(98 downto 35) + c3_2;
			
			-- Koeffizeinten b0 und b2 der Kaskade 3 berechnen
			when x"10" => g <= g3;
			when x"12" => mul_a <= c2_3; mul_b <= v;
			when x"15" => b0_3 <= c1_3 + mul_r(98 downto 35) + c3_3; b2_3 <= c1_3 - mul_r(98 downto 35) + c3_3;
			
			-- Koeffizeinten b0 und b2 der Kaskade 4 berechnen
			when x"1A" => g <= g4;
			when x"1C" => mul_a <= c2_4; mul_b <= v;
			when x"1F" => b0_4 <= c1_4 + mul_r(98 downto 35) + c3_4; b2_4 <= c1_4 - mul_r(98 downto 35) + c3_4;
			
			-- Koeffizeinten b0 und b2 der Kaskade 5 berechnen
			when x"20" => g <= g5;
			when x"22" => mul_a <= c2_5; mul_b <= v;
			when x"25" => b0_5 <= c1_5 + mul_r(98 downto 35) + c3_5; b2_5 <= c1_5 - mul_r(98 downto 35) + c3_5;

			-- Koeffizeinten b0 und b2 der Kaskade 6 berechnen
			when x"2A" => g <= g6;
			when x"2C" => mul_a <= c2_6; mul_b <= v;
			when x"2F" => b0_6 <= c1_6 + mul_r(98 downto 35) + c3_6; b2_6 <= c1_6 - mul_r(98 downto 35) + c3_6;

			-- Koeffizeinten b0 und b2 der Kaskade 7 berechnen
			when x"30" => g <= g7;
			when x"32" => mul_a <= c2_7; mul_b <= v;
			when x"35" => b0_7 <= c1_7 + mul_r(98 downto 35) + c3_7; b2_7 <= c1_7 - mul_r(98 downto 35) + c3_7;

			-- Koeffizeinten b0 und b2 der Kaskade 8 berechnen
			when x"3A" => g <= g8;
			when x"3C" => mul_a <= c2_8; mul_b <= v;
			when x"3F" => b0_8 <= c1_8 + mul_r(98 downto 35) + c3_8; b2_8 <= c1_8 - mul_r(98 downto 35) + c3_8;

			-- Koeffizeinten b0 und b2 der Kaskade 9 berechnen
			when x"40" => g <= g9;
			when x"42" => mul_a <= c2_9; mul_b <= v;
			when x"45" => b0_9 <= c1_9 + mul_r(98 downto 35) + c3_9; b2_9 <= c1_9 - mul_r(98 downto 35) + c3_9;

			-- Koeffizeinten b0 und b2 der Kaskade 10 berechnen
			when x"4A" => g <= g10;
			when x"4C" => mul_a <= c2_10; mul_b <= v;
			when x"4F" => b0_10 <= c1_10 + mul_r(98 downto 35) + c3_10; b2_10 <= c1_10 - mul_r(98 downto 35) + c3_10;

			-- Koeffizeinten b0 und b2 der Kaskade 11 berechnen
			when x"50" => g <= g11;
			when x"52" => mul_a <= c2_11; mul_b <= v;
			when x"55" => b0_11 <= c1_11 + mul_r(98 downto 35) + c3_11; b2_11 <= c1_11 - mul_r(98 downto 35) + c3_11;

			-- Koeffizeinten b0 und b2 der Kaskade 12 berechnen
			when x"5A" => g <= g12;
			when x"5C" => mul_a <= c2_12; mul_b <= v;
			when x"5F" => b0_12 <= c1_12 + mul_r(98 downto 35) + c3_12; b2_12 <= c1_12 - mul_r(98 downto 35) + c3_12;

			-- Koeffizeinten b0 und b2 der Kaskade 13 berechnen
			when x"60" => g <= g13;
			when x"62" => mul_a <= c2_13; mul_b <= v;
			when x"65" => b0_13 <= c1_13 + mul_r(98 downto 35) + c3_13; b2_13 <= c1_13 - mul_r(98 downto 35) + c3_13;

			-- Koeffizeinten b0 und b2 der Kaskade 14 berechnen
			when x"6A" => g <= g14;
			when x"6C" => mul_a <= c2_14; mul_b <= v;
			when x"6F" => b0_14 <= c1_14 + mul_r(98 downto 35) + c3_14; b2_14 <= c1_14 - mul_r(98 downto 35) + c3_14;
			
			when others => null;
			
		end case;
		
		if (counter < x"70") then counter <= counter + 1; else counter <= x"00"; end if;
		
	end process;
	

end behavior;




