------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  db2linear.vhd                                  --
--  Version 2012-06-09-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity db2linear is
	port 
	(
		db 		: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor
		linear 	: out signed(39 downto 0)	-- linearer Verstärkungsfaktor
	);
end db2linear;



architecture behavior of db2linear is
begin

	-- Look-Up-Table zur Umrechnung logarithmischer 
	-- Verstärkungsfaktor zu linearen Verstärkungsfaktor
	-- Format des linearen Verstärkungsfaktor ist 
	-- Vorzeichenbehaftetes 2er Komplement mit 
	-- 4 Vorkommastellen und 35 Nachkommastellen
	with db select	linear <=	x"5000000000" when x"14",
										x"474CD1B7F1" when x"13",
										x"3F8BD79D82" when x"12",
										x"38A2BACB45" when x"11",
										x"327A01A469" when x"10",
										x"2CFCC01646" when x"0F",
										x"2818508600" when x"0E",
										x"23BC1478BB" when x"0D",
										x"1FD93C1F52" when x"0C",
										x"1C629405AB" when x"0B",
										x"194C583ADA" when x"0A",
										x"168C0C59AB" when x"09",
										x"141857E9D4" when x"08",
										x"11E8E6A0D8" when x"07",
										x"0FF64C16AD" when x"06",
										x"0E39EA8E2A" when x"05",
										x"0CADDC7B6A" when x"04",
										x"0B4CE07BF4" when x"03",
										x"0A12477C7E" when x"02",
										x"08F9E4CFB5" when x"01",
										x"0800000000" when x"00",
										x"0721482BFE" when x"FF",
										x"065AC8C2F3" when x"FE",
										x"05A9DF7ABA" when x"FD",
										x"050C335D3D" when x"FC",
										x"047FACCF07" when x"FB",
										x"04026E73CC" when x"FA",
										x"0392CED8DF" when x"F9",
										x"032F52CFEE" when x"F8",
										x"02D6A866F7" when x"F7",
										x"0287A26C49" when x"F6",
										x"0241346F5D" when x"F5",
										x"02026F30FB" when x"F4",
										x"01CA7D767C" when x"F3",
										x"0198A13577" when x"F2",
										x"016C310E37" when x"F1",
										x"0144960C57" when x"F0",
										x"012149A5FE" when x"EF",
										x"0101D3F2D9" when x"EE",
										x"00E5CA14C5" when x"ED",
										x"00CCCCCCCC" when x"EC",
										x"00B6873799" when x"EB",
										x"00A2ADAD18" when x"EA",
										x"0090FCBF79" when x"E9",
										x"008138561F" when x"E8",
										x"00732AE180" when x"E7",
										x"0066A4A52E" when x"E6",
										x"005B7B15AF" when x"E5",
										x"00518847FE" when x"E4",
										x"0048AA70B2" when x"E3",
										x"0040C3713A" when x"E2",
										x"0039B87189" when x"E1",
										x"00337184E5" when x"E0",
										x"002DD958A6" when x"DF",
										x"0028DCEBBF" when x"DE",
										x"00246B4E38" when x"DD",
										x"00207567A2" when x"DC",
										x"001CEDC3CC" when x"DB",
										x"0019C86515" when x"DA",
										x"0016FA9BAD" when x"D9",
										x"00147AE147" when x"D8",
										x"0800000000" when others;
										
end behavior;
