------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  filter_unit.vhd                                 --
--  Version 2012-06-15-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
USE ieee.std_logic_1164.all;
use ieee.numeric_std.all;


--  Entity Declaration
entity filter_unit is
	port 
	(
		-- inputs
		clk_50MHz	: in std_logic;					-- 50 MHz Takteingang
		dl				: in signed(23 downto 0);		-- Dateneingang linker Kanal
		dr				: in signed(23 downto 0);		-- Dateneingang rechter Kanal
		enl 			: in std_logic;					-- Freigabesignal zur Übernahme der Daten am Dateneingang linker Kanal
		enr 		 	: in std_logic;					-- Freigabesignal zur Übernahme der Daten am Dateneingang rechter Kanal
		vol		  	: in signed(7 downto 0);		-- Verstärkungsfaktor Lauftstärke in dB				
		g1 		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g2			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g3			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g4			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g5			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g6			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g7			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g8			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g9 		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g10		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g11		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g12 			: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g13		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
		g14 			: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB
		bypass		: in std_logic;					-- Bypass Signaleingang (0 -> EQ ein; 1 -> EQ aus)
	
		-- outputs
		qdl		: out signed(23 downto 0);			-- Datenausgang linker Kanal		
		qdr		: out signed(23 downto 0);			-- Datenausgang rechter Kanal
		enql		: out std_logic;						-- Freigabesignal zur Übernahme der Daten am Datenausgang linker Kanal	
		enqr		: out std_logic						-- Freigabesignal zur Übernahme der Daten am Datenausgang rechter Kanal	
	);
end filter_unit;



architecture behavior of filter_unit is

	signal idl 				: signed(23 downto 0) 	:= (others => '0');		-- Zwischenspeicher Eingangsdaten linker Kanal
	signal idr 				: signed(23 downto 0) 	:= (others => '0');		-- Zwischenspeicher Eingangsdaten rechter Kanal
	signal idn 				: signed(23 downto 0) 	:= (others => '0');		-- Zwischenspeicher für Zwischenergebnisse
	signal idm 				: signed(23 downto 0) 	:= (others => '0');		-- Zwischenspeicher für Zwischenergebnisse
	signal counter_left  : unsigned(11 downto 0) := (others => '0');		-- Zähler für workflow linker Kanal
	signal counter_right : unsigned(11 downto 0) := (others => '0'); 		-- Zähler für workflow rechter Kanal
	signal en 				: std_logic 				:= '0';						-- Freigabesignal zur Übernahme der Daten IIR-Filter
	signal yn 				: signed(23 downto 0) 	:= (others => '0');		-- Ergebnis der Berechnung des IIR-Filters
	signal volume 			: signed(39 downto 0) 	:= x"0800000000";			-- linearer Verstärkungsfaktor Lautstärke

	-- IIR-Filter Komponente
	component iir
		port
		(
			-- inputs
			clk_50MHz	: in std_logic;				-- 50 MHz Takt
			x				: in signed(23 downto 0);	-- Audiosample Eingang
			en				: in std_logic;				-- Freigabesignal zum Einlesen der Eingangsdaten 
			b0_1  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  1 des IIR-Filters
			b2_1  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  1 des IIR-Filters
			b0_2  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  2 des IIR-Filters
			b2_2  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  2 des IIR-Filters
			b0_3  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  3 des IIR-Filters
			b2_3  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  3 des IIR-Filters
			b0_4  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  4 des IIR-Filters
			b2_4  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  4 des IIR-Filters
			b0_5  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  5 des IIR-Filters
			b2_5  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  5 des IIR-Filters
			b0_6  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  6 des IIR-Filters
			b2_6  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  6 des IIR-Filters
			b0_7  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  7 des IIR-Filters
			b2_7  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  7 des IIR-Filters
			b0_8  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  8 des IIR-Filters
			b2_8  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  8 des IIR-Filters
			b0_9  		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  9 des IIR-Filters
			b2_9  		: in signed(63 downto 0);	-- Koeffizient b2 für Filter  9 des IIR-Filters
			b0_10 		: in signed(63 downto 0);	-- Koeffizient b0 für Filter  0 des IIR-Filters
			b2_10 		: in signed(63 downto 0);	-- Koeffizient b2 für Filter 10 des IIR-Filters
			b0_11 		: in signed(63 downto 0);	-- Koeffizient b0 für Filter 11 des IIR-Filters
			b2_11 		: in signed(63 downto 0);	-- Koeffizient b2 für Filter 11 des IIR-Filters
			b0_12 		: in signed(63 downto 0);	-- Koeffizient b0 für Filter 12 des IIR-Filters
			b2_12 		: in signed(63 downto 0);	-- Koeffizient b2 für Filter 12 des IIR-Filters
			b0_13 		: in signed(63 downto 0);	-- Koeffizient b0 für Filter 13 des IIR-Filters
			b2_13 		: in signed(63 downto 0);	-- Koeffizient b2 für Filter 13 des IIR-Filters
			b0_14 		: in signed(63 downto 0);	-- Koeffizient b0 für Filter 14 des IIR-Filters
			b2_14 		: in signed(63 downto 0);	-- Koeffizient b2 für Filter 14 des IIR-Filters
		
			-- outputs
			y : out signed(23 downto 0) -- Audiosample Ausgang (gefiltert)		
		);
	end component;
	
	-- Komponente zur Umrechnung des logarithmischen Verstärkungsfaktor in den linearer Verstärkungsfaktor (LUT)
	component db2linear
		port 
		(
			db 		: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor
			linear 	: out signed(39 downto 0)	-- linearer Verstärkungsfaktor
		);
	end component;
	
	-- Multiplizierer zur Lautstärkeberechnung  (Megafunctions LPM_MULT)
	component lpm_mult_volume
		port
		(
			dataa		: IN signed (23 DOWNTO 0);		
			datab		: IN signed (39 DOWNTO 0);
			result	: OUT signed (63 DOWNTO 0)
		);
	end component;
	
	-- Signale zur Übergabe und Übernahme der Operatoren und 
	-- des Ergebnisses der Multiplikation (Lautstärke)
	signal vma : signed(23 downto 0) := (others => '0');
	signal vmb : signed(39 downto 0) := (others => '0');
	signal vmr : signed(63 downto 0) := (others => '0');
	
	-- Koeffizientenregister für die Koeffizienten b0 und b2 des IIR-Filters.
	-- Im Register sind die Koeffizienten nicht statisch hinterlegt (LUT), sondern
	-- werden im Register je nach Verstärkungsfaktor für die jeweilige Kaskade berechnet.
	component coefficients_register
		port
		(
			-- inputs
			clk_50MHz	: in std_logic;				-- 50 MHz Takt
			g1  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 1
			g2  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 2				
			g3  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 3				
			g4  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 4				
			g5  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 5				
			g6  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 6				
			g7  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 7				
			g8  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 8				
			g9  			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 9				
			g10 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 10				
			g11 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 11				
			g12 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 12				
			g13 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 13				
			g14 			: in signed(7 downto 0);	-- logarithmischer Verstärkungsfaktor Filter 14				
		
			-- outputs
			b0_1  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  1 des IIR-Filters
			b2_1  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  1 des IIR-Filters
			b0_2  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  2 des IIR-Filters
			b2_2  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  2 des IIR-Filters
			b0_3  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  3 des IIR-Filters
			b2_3  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  3 des IIR-Filters
			b0_4  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  4 des IIR-Filters
			b2_4  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  4 des IIR-Filters
			b0_5  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  5 des IIR-Filters
			b2_5  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  5 des IIR-Filters
			b0_6  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  6 des IIR-Filters
			b2_6  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  6 des IIR-Filters
			b0_7  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  7 des IIR-Filters
			b2_7  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  7 des IIR-Filters
			b0_8  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  8 des IIR-Filters
			b2_8  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  8 des IIR-Filters
			b0_9  		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  9 des IIR-Filters
			b2_9  		: out signed(63 downto 0);	-- Koeffizient b2 für Filter  9 des IIR-Filters
			b0_10 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter  0 des IIR-Filters
			b2_10 		: out signed(63 downto 0);	-- Koeffizient b2 für Filter 10 des IIR-Filters
			b0_11 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter 11 des IIR-Filters
			b2_11 		: out signed(63 downto 0);	-- Koeffizient b2 für Filter 11 des IIR-Filters
			b0_12 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter 12 des IIR-Filters
			b2_12 		: out signed(63 downto 0);	-- Koeffizient b2 für Filter 12 des IIR-Filters
			b0_13 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter 13 des IIR-Filters
			b2_13 		: out signed(63 downto 0);	-- Koeffizient b2 für Filter 13 des IIR-Filters
			b0_14 		: out signed(63 downto 0);	-- Koeffizient b0 für Filter 14 des IIR-Filters
			b2_14 		: out signed(63 downto 0)	-- Koeffizient b2 für Filter 14 des IIR-Filters
		);
	end component;
	
	-- Signale zur Übernahme bzw. Übergabe der Koeffizienten
	signal b0_1  : signed(63 downto 0);
	signal b2_1  : signed(63 downto 0);
	signal b0_2  : signed(63 downto 0);
	signal b2_2  : signed(63 downto 0);
	signal b0_3  : signed(63 downto 0);
	signal b2_3  : signed(63 downto 0);
	signal b0_4  : signed(63 downto 0);
	signal b2_4  : signed(63 downto 0);
	signal b0_5  : signed(63 downto 0);
	signal b2_5  : signed(63 downto 0);
	signal b0_6  : signed(63 downto 0);
	signal b2_6  : signed(63 downto 0);
	signal b0_7  : signed(63 downto 0);
	signal b2_7  : signed(63 downto 0);
	signal b0_8  : signed(63 downto 0);
	signal b2_8  : signed(63 downto 0);
	signal b0_9  : signed(63 downto 0);
	signal b2_9  : signed(63 downto 0);
	signal b0_10 : signed(63 downto 0);
	signal b2_10 : signed(63 downto 0);
	signal b0_11 : signed(63 downto 0);
	signal b2_11 : signed(63 downto 0);
	signal b0_12 : signed(63 downto 0);
	signal b2_12 : signed(63 downto 0);
	signal b0_13 : signed(63 downto 0);
	signal b2_13 : signed(63 downto 0);
	signal b0_14 : signed(63 downto 0);
	signal b2_14 : signed(63 downto 0);

begin
	
	-- Umrechnung über LUT des logarithmischen Verstärkungsfaktors 
	-- der Lautstärke in den linearen
	volume_db2linear : db2linear port map (vol, volume);
	
	-- Multiplizierer zur Berechnung des neuen Lautstärkepegels
	volume_muliplier : lpm_mult_volume port map (vma, vmb, vmr);
	
	-- Process zur Berechnung des Audio-Ausgangssignals
	process	
	begin
	
		-- Warten auf steigende Flanke des 50 MHz Taktes
		wait until rising_edge(clk_50MHz);

		-- workflow-counter zurücksetzen bei entsprechendem Freigabesignal
		if (enl='0') then	counter_left <= (others => '0');	end if;
		if (enr='0') then	counter_right <= (others => '0'); end if;
		
		-- Berechnung des linken Audiosignals
		case counter_left is
			when x"000" => idl <= dl;	-- Übernahme des Audiosignals vom audio_input_register
			when x"001" => vma <= idl; vmb <= volume;	-- Berechnungs Lautstärke
			when x"006" => idn <= vmr(58 downto 35); en <= '0'; -- Übergabe des Ergebnisses der Lautstärkeberechnung an den IIR-Filter
			when x"007" => en <= '1'; -- Freigabeimpuls zur Übernahme des Audiosignals im IIR-Filter zurücksetzen
			when x"090" => if (bypass='0') then idm <= yn; else idm <= idn; end if;	-- Bypass Prüfung. (0 -> Filterergebnis verwenden, 1 -> Ergebnis Lautstärkeberechnung verwenden)
			when x"091" => qdl <= idm; enql <= '0'; -- Audio-Ausgangssignal an den Ausgang legen und Freigabeimpuls an audio_output_register senden
			when x"092" => enql <= '1'; -- Freigabeimpuls zurücksetzen
			when others => null;
		end case;
		if counter_left < x"093" then counter_left <= counter_left + 1; end if;

		-- Berechnung des rechten Audiosignals
		case counter_right is
			when x"000" => idr <= dr;	-- Übernahme des Audiosignals vom audio_input_register
			when x"001" => vma <= idr; vmb <= volume;	-- Berechnungs Lautstärke
			when x"006" => idn <= vmr(58 downto 35); en <= '0';	-- Übergabe des Ergebnisses der Lautstärkeberechnung an den IIR-Filter
			when x"007" => en <= '1';	-- Freigabeimpuls zur Übernahme des Audiosignals im IIR-Filter zurücksetzen
			when x"090" => if (bypass='0') then idm <= yn; else idm <= idn; end if;	-- Bypass Prüfung. (0 -> Filterergebnis verwenden, 1 -> Ergebnis Lautstärkeberechnung verwenden)
			when x"091" => qdr <= idm; enqr <= '0';	-- Audio-Ausgangssignal an den Ausgang legen und Freigabeimpuls an audio_output_register senden
			when x"092" => enqr <= '1';	-- Freigabeimpuls zurücksetzen
			when others => null;
		end case;
		if counter_right < x"093" then counter_right <= counter_right + 1; end if;

	end process;

	-- Koeffizientenregister Ports zuweisen
	coefficients_register_1 : coefficients_register port map
	(
		clk_50MHz,
		g1,
		g2,
		g3,
		g4,
		g5,
		g6,
		g7,
		g8,
		g9,
		g10,
		g11,
		g12,
		g13,
		g14,
		b0_1,
		b2_1,
		b0_2,
		b2_2,
		b0_3,
		b2_3,
		b0_4,
		b2_4,
		b0_5,
		b2_5,
		b0_6,
		b2_6,
		b0_7,
		b2_7,
		b0_8,
		b2_8,
		b0_9,
		b2_9,
		b0_10,
		b2_10,
		b0_11,
		b2_11,
		b0_12,
		b2_12,
		b0_13,
		b2_13,
		b0_14,
		b2_14
	);

	-- IIR Filter Ports zuweisen
	filter: iir port map
	(
		clk_50MHz,
		idn,
		en,
		b0_1,
		b2_1,
		b0_2,
		b2_2,
		b0_3,
		b2_3,
		b0_4,
		b2_4,
		b0_5,
		b2_5,
		b0_6,
		b2_6,
		b0_7,
		b2_7,
		b0_8,
		b2_8,
		b0_9,
		b2_9,
		b0_10,
		b2_10,
		b0_11,
		b2_11,
		b0_12,
		b2_12,
		b0_13,
		b2_13,
		b0_14,
		b2_14,
		yn
	);
	
end behavior;
