------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  mmi_controller.vhd                              --
--  Version 2012-06-13-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity mmi_controller is
	port
	(
			-- inputs
			clk_50MHz	 	: in std_logic;		-- 50 MHz Takt
			next_filter	 	: in std_logic;		-- nächsten Fitler auswählen
			prior_filter 	: in std_logic;		-- vorherigen Filter ausählen
			inc				: in std_logic; 		-- Verstärkungsfaktor Filter/Volume inkrementieren (1dB)
			dec				: in std_logic;		-- Verstärkungsfaktor Filter/Volume inkrementieren (1dB)
			seg_off			: in std_logic;		-- 7-Segement-Anzeigen ein-/ausschalten (0 -> ein, 1 -> aus);
			reset				: in std_logic;		-- Rücksetzeingang
			jd					: in std_logic;

			-- outputs
			lcd_1			: out unsigned(11 downto 0) := x"120";			-- Zeichen 1 zur Anzeige des ausgewähltn Filters
			lcd_2			: out unsigned(11 downto 0) := x"120";			-- Zeichen 2 zur Anzeige des ausgewähltn Filters
			lcd_3			: out unsigned(11 downto 0) := x"120";			-- Zeichen 3 zur Anzeige des ausgewähltn Filters
			lcd_4			: out unsigned(11 downto 0) := x"120";			-- Vorzeichen des Verstärkungsfaktors
			lcd_5			: out unsigned(11 downto 0) := x"120";			-- Zehner-Stelle des Verstärkungsfaktors in dB
			lcd_6			: out unsigned(11 downto 0) := x"120";			-- Einer-Stelle des Verstärkungsfaktors in dB
			vol_value	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Lautstärke in dB
			f1_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 1 in dB
			f2_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 2 in dB
			f3_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 3 in dB
			f4_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 4 in dB
			f5_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 5 in dB
			f6_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 6 in dB
			f7_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 7 in dB
			f8_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 8 in dB
			f9_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 9 in dB
			f10_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 10 in dB
			f11_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 11 in dB
			f12_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 12 in dB
			f13_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 13 in dB
			f14_value 	: out signed(7 downto 0) := (others => '0');	-- Verstärkungsfaktor Filter 14 in dB
			filter_ones	: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Auswahlfilter Einer-Stelle
			filter_ten	: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Auswahlfilter Zehner-Stelle
			db_ones		: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Einer-Stelle
			db_ten		: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Zehner-Stelle
			db_sign		: out unsigned(6 downto 0);						-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Vorzeichen
			db_letter_1 : out unsigned(6 downto 0);						-- 7-Segement-Anzeige Buchstebe 'd' für db
			db_letter_2 : out unsigned(6 downto 0)							-- 7-Segement-Anzeige Buchstebe 'b' für db
	);
end mmi_controller;




architecture behavior of mmi_controller is
	
	-- Signale deklarieren
	signal filter		: unsigned(3 downto 0) := x"0";					-- ausgewählter Filter	
	signal vol_db		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Lautstärke in dB
	signal f1_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter1 in dB
	signal f2_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter2 in dB
	signal f3_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter3 in dB
	signal f4_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter4 in dB
	signal f5_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter5 in dB
	signal f6_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter6 in dB
	signal f7_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter7 in dB
	signal f8_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter8 in dB
	signal f9_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter9 in dB
	signal f10_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter10 in dB
	signal f11_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter11 in dB
	signal f12_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter12 in dB
	signal f13_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter13 in dB
	signal f14_db 		: signed(7 downto 0) := (others => '0');		-- Zwischenspeicher Verstärkungsfaktor Filter14 in dB
	signal clk_counter 	: unsigned(27 downto 0) := x"0000000";
	
begin
	
	-- count_clk -> internen Zähler nach 10 ms weiterschalten (interne Takterzeugung 100 Hz)
	count_clk: process
	begin
		wait until rising_edge(clk_50MHz);
		if reset='1' then 
			clk_counter <= x"0000000";
		else 
			if (clk_counter < x"05F5E10") then 
				clk_counter <= clk_counter + 1;
			else 
				clk_counter <= x"0000000";
			end if;
		end if;
	end process;
	
	-- set_filter -> Bei Betätigung der Taste KEY[3] bzw. KEY[2]
	-- Filterauswahl (filter) inkrementieren bzw. dekrementieren
	set_filter: process
	begin
		wait until rising_edge(clk_50MHz);
		if (reset='1') then 
			filter <= x"0";
		else 
			if (clk_counter = x"05F5E10") then 
				if (next_filter='0') then
					if (filter = x"E") then
						filter <= x"0";
					else
						filter <= filter + 1;
					end if;
				elsif (prior_filter = '0') then
					if (filter = x"0") then
						filter <= x"E";
					else 	
						filter <= filter - 1;
					end if;
				end if;
			end if;
		end if;
	end process set_filter;
	
	-- set_value -> Bei Betätigung der Taste KEY[1] bzw. KEY[0]
	-- Verstärkungsfaktor (vol_db, f1_db ...) inkrementieren bzw. dekrementieren
	-- Bei einem Reset werden alle Werte auf 0 dB gesetzt.
	set_value: process
	variable tmp : signed(7 downto 0);
	begin
		wait until rising_edge(clk_50MHz);
		if (reset='1') then 
			vol_db  <= (others => '0');
			f1_db   <= (others => '0');
			f2_db   <= (others => '0');
			f3_db   <= (others => '0');
			f4_db   <= (others => '0');
			f5_db   <= (others => '0');
			f6_db   <= (others => '0');
			f7_db   <= (others => '0');
			f8_db   <= (others => '0');
			f9_db   <= (others => '0');
			f10_db  <= (others => '0');
			f11_db  <= (others => '0');
			f12_db  <= (others => '0');
			f13_db  <= (others => '0');
			f14_db  <= (others => '0');
		else 
			if (jd='1') then
				vol_db  <= x"00";
				f1_db   <= x"06";
				f2_db   <= x"06";
				f3_db   <= x"03";
				f4_db   <= x"00";
				f5_db   <= x"00";
				f6_db   <= x"FE";
				f7_db   <= x"FD";
				f8_db   <= x"FE";
				f9_db   <= x"00";
				f10_db  <= x"00";
				f11_db  <= x"03";
				f12_db  <= x"03";
				f13_db  <= x"06";
				f14_db  <= x"06";
			end if;
			if (clk_counter = x"05F5E10") then 
				if (inc='0') then
					case filter is
						when x"0" => if (vol_db = x"14") then vol_db <= vol_db; else vol_db <= vol_db + 1;	end if;
						when x"1" => if (f1_db  = x"14") then f1_db  <= f1_db; else f1_db   <= f1_db  + 1;	end if;
						when x"2" => if (f2_db  = x"14") then f2_db  <= f2_db; else f2_db   <= f2_db  + 1;	end if;
						when x"3" => if (f3_db  = x"14") then f3_db  <= f3_db; else f3_db   <= f3_db  + 1;	end if;
						when x"4" => if (f4_db  = x"14") then f4_db  <= f4_db; else f4_db   <= f4_db  + 1;	end if;
						when x"5" => if (f5_db  = x"14") then f5_db  <= f5_db; else f5_db   <= f5_db  + 1;	end if;
						when x"6" => if (f6_db  = x"14") then f6_db  <= f6_db; else f6_db   <= f6_db  + 1;	end if;
						when x"7" => if (f7_db  = x"14") then f7_db  <= f7_db; else f7_db   <= f7_db  + 1;	end if;
						when x"8" => if (f8_db  = x"14") then f8_db  <= f8_db; else f8_db   <= f8_db  + 1;	end if;
						when x"9" => if (f9_db  = x"14") then f9_db  <= f9_db; else f9_db   <= f9_db  + 1;	end if;
						when x"A" => if (f10_db = x"14") then f10_db <= f10_db; else f10_db <= f10_db + 1;	end if;
						when x"B" => if (f11_db = x"14") then f11_db <= f11_db; else f11_db <= f11_db + 1;	end if;
						when x"C" => if (f12_db = x"14") then f12_db <= f12_db; else f12_db <= f12_db + 1;	end if;
						when x"D" => if (f13_db = x"14") then f13_db <= f13_db; else f13_db <= f13_db + 1;	end if;
						when x"E" => if (f14_db = x"14") then f14_db <= f14_db; else f14_db <= f14_db + 1;	end if;
						when others => null;
					end case;
				elsif (dec='0') then
					case filter is
						when x"0" => if (vol_db = x"D8") then vol_db <= vol_db; else vol_db <= vol_db - 1;	end if;
						when x"1" => if (f1_db  = x"D8") then f1_db  <= f1_db;  else f1_db   <= f1_db  - 1;	end if;
						when x"2" => if (f2_db  = x"D8") then f2_db  <= f2_db;  else f2_db   <= f2_db  - 1;	end if;
						when x"3" => if (f3_db  = x"D8") then f3_db  <= f3_db;  else f3_db   <= f3_db  - 1;	end if;
						when x"4" => if (f4_db  = x"D8") then f4_db  <= f4_db;  else f4_db   <= f4_db  - 1;	end if;
						when x"5" => if (f5_db  = x"D8") then f5_db  <= f5_db;  else f5_db   <= f5_db  - 1;	end if;
						when x"6" => if (f6_db  = x"D8") then f6_db  <= f6_db;  else f6_db   <= f6_db  - 1;	end if;
						when x"7" => if (f7_db  = x"D8") then f7_db  <= f7_db;  else f7_db   <= f7_db  - 1;	end if;
						when x"8" => if (f8_db  = x"D8") then f8_db  <= f8_db;  else f8_db   <= f8_db  - 1;	end if;
						when x"9" => if (f9_db  = x"D8") then f9_db  <= f9_db;  else f9_db   <= f9_db  - 1;	end if;
						when x"A" => if (f10_db = x"D8") then f10_db <= f10_db; else f10_db  <= f10_db - 1;	end if;
						when x"B" => if (f11_db = x"D8") then f11_db <= f11_db; else f11_db  <= f11_db - 1;	end if;
						when x"C" => if (f12_db = x"D8") then f12_db <= f12_db; else f12_db  <= f12_db - 1;	end if;
						when x"D" => if (f13_db = x"D8") then f13_db <= f13_db; else f13_db  <= f13_db - 1;	end if;
						when x"E" => if (f14_db = x"D8") then f14_db <= f14_db; else f14_db  <= f14_db - 1;	end if;
						when others => null;
					end case;
				end if;
			end if;
		end if;
	end process set_value;
	
	
	-- set_7seg -> Ausgabe der Werte über die 7 - Segment - Anzeigen
	set_7seg: process
	variable tmp : signed(7 downto 0);
	begin
		
		-- Warten auf steigende Flanke des 50 MHz Taktes
		wait until falling_edge(clk_50MHz);
		
		-- Abfrage ob die 7-Segment-Anzeige ein- oder ausgeschaltet ist
		-- und entsprechend die Werte für die einzelnen Segment-Anzeigen setzen
		if (seg_off='0') then
			
			-- Filterauswahl anzeigen
			case filter is
				when x"0" => tmp := vol_db; filter_ten <= "1000000"; filter_ones <= "1000000";
				when x"1" => tmp := f1_db;  filter_ten <= "1000000"; filter_ones <= "1111001";
				when x"2" => tmp := f2_db;  filter_ten <= "1000000"; filter_ones <= "0100100";
				when x"3" => tmp := f3_db;  filter_ten <= "1000000"; filter_ones <= "0110000";
				when x"4" => tmp := f4_db;  filter_ten <= "1000000"; filter_ones <= "0011001";
				when x"5" => tmp := f5_db;  filter_ten <= "1000000"; filter_ones <= "0010010";
				when x"6" => tmp := f6_db;  filter_ten <= "1000000"; filter_ones <= "0000010";
				when x"7" => tmp := f7_db;  filter_ten <= "1000000"; filter_ones <= "1111000";
				when x"8" => tmp := f8_db;  filter_ten <= "1000000"; filter_ones <= "0000000";
				when x"9" => tmp := f9_db;  filter_ten <= "1000000"; filter_ones <= "0010000";
				when x"A" => tmp := f10_db; filter_ten <= "1111001"; filter_ones <= "1000000";
				when x"B" => tmp := f11_db; filter_ten <= "1111001"; filter_ones <= "1111001";
				when x"C" => tmp := f12_db; filter_ten <= "1111001"; filter_ones <= "0100100";
				when x"D" => tmp := f13_db; filter_ten <= "1111001"; filter_ones <= "0110000";
				when x"E" => tmp := f14_db; filter_ten <= "1111001"; filter_ones <= "0011001";
				when others => null;
			end case;
			
			-- Vorzeichen des Verstärkungsfaktors ermitteln und Vorzeichenanzeige setzen
			if (tmp(7) = '1')  then
				db_sign <= "0111111";
			else
				db_sign <= "1111111";
			end if;
			
			-- Verstärkungsfaktor anzeigen
			case tmp is
				
				when x"14" => db_ten <= "0100100"; db_ones <= "1000000";

				when x"0B" => db_ten <= "1111001"; db_ones <= "1111001";
				when x"0C" => db_ten <= "1111001"; db_ones <= "0100100";
				when x"0D" => db_ten <= "1111001"; db_ones <= "0110000";
				when x"0E" => db_ten <= "1111001"; db_ones <= "0011001";
				when x"0F" => db_ten <= "1111001"; db_ones <= "0010010";
				when x"10" => db_ten <= "1111001"; db_ones <= "0000010";
				when x"11" => db_ten <= "1111001"; db_ones <= "1111000";
				when x"12" => db_ten <= "1111001"; db_ones <= "0000000";
				when x"13" => db_ten <= "1111001"; db_ones <= "0010000";
				when x"0A" => db_ten <= "1111001"; db_ones <= "1000000";

				when x"01" => db_ten <= "1000000"; db_ones <= "1111001";
				when x"02" => db_ten <= "1000000"; db_ones <= "0100100";
				when x"03" => db_ten <= "1000000"; db_ones <= "0110000";
				when x"04" => db_ten <= "1000000"; db_ones <= "0011001";
				when x"05" => db_ten <= "1000000"; db_ones <= "0010010";
				when x"06" => db_ten <= "1000000"; db_ones <= "0000010";
				when x"07" => db_ten <= "1000000"; db_ones <= "1111000";
				when x"08" => db_ten <= "1000000"; db_ones <= "0000000";
				when x"09" => db_ten <= "1000000"; db_ones <= "0010000";
				when x"00" => db_ten <= "1000000"; db_ones <= "1000000";

				when x"FF" => db_ten <= "1000000"; db_ones <= "1111001";
				when x"FE" => db_ten <= "1000000"; db_ones <= "0100100";
				when x"FD" => db_ten <= "1000000"; db_ones <= "0110000";
				when x"FC" => db_ten <= "1000000"; db_ones <= "0011001";
				when x"FB" => db_ten <= "1000000"; db_ones <= "0010010";
				when x"FA" => db_ten <= "1000000"; db_ones <= "0000010";
				when x"F9" => db_ten <= "1000000"; db_ones <= "1111000";
				when x"F8" => db_ten <= "1000000"; db_ones <= "0000000";
				when x"F7" => db_ten <= "1000000"; db_ones <= "0010000";
				
				when x"F6" => db_ten <= "1111001"; db_ones <= "1000000";
				when x"F5" => db_ten <= "1111001"; db_ones <= "1111001";
				when x"F4" => db_ten <= "1111001"; db_ones <= "0100100";
				when x"F3" => db_ten <= "1111001"; db_ones <= "0110000";
				when x"F2" => db_ten <= "1111001"; db_ones <= "0011001";
				when x"F1" => db_ten <= "1111001"; db_ones <= "0010010";
				when x"F0" => db_ten <= "1111001"; db_ones <= "0000010";
				when x"EF" => db_ten <= "1111001"; db_ones <= "1111000";
				when x"EE" => db_ten <= "1111001"; db_ones <= "0000000";
				when x"ED" => db_ten <= "1111001"; db_ones <= "0010000";
				
				when x"EC" => db_ten <= "0100100"; db_ones <= "1000000";
				when x"EB" => db_ten <= "0100100"; db_ones <= "1111001";
				when x"EA" => db_ten <= "0100100"; db_ones <= "0100100";
				when x"E9" => db_ten <= "0100100"; db_ones <= "0110000";
				when x"E8" => db_ten <= "0100100"; db_ones <= "0011001";
				when x"E7" => db_ten <= "0100100"; db_ones <= "0010010";
				when x"E6" => db_ten <= "0100100"; db_ones <= "0000010";
				when x"E5" => db_ten <= "0100100"; db_ones <= "1111000";
				when x"E4" => db_ten <= "0100100"; db_ones <= "0000000";
				when x"E3" => db_ten <= "0100100"; db_ones <= "0010000";

				when x"E2" => db_ten <= "0110000"; db_ones <= "1000000";
				when x"E1" => db_ten <= "0110000"; db_ones <= "1111001";
				when x"E0" => db_ten <= "0110000"; db_ones <= "0100100";
				when x"DF" => db_ten <= "0110000"; db_ones <= "0110000";
				when x"DE" => db_ten <= "0110000"; db_ones <= "0011001";
				when x"DD" => db_ten <= "0110000"; db_ones <= "0010010";
				when x"DC" => db_ten <= "0110000"; db_ones <= "0000010";
				when x"DB" => db_ten <= "0110000"; db_ones <= "1111000";
				when x"DA" => db_ten <= "0110000"; db_ones <= "0000000";
				when x"D9" => db_ten <= "0110000"; db_ones <= "0010000";
				
				when x"D8" => db_ten <= "0011001"; db_ones <= "1000000";
				when others => null;
			end case;
			db_letter_1 <= "0100001";
			db_letter_2 <= "0000011";
		else
			-- Sollen die 7-Segementanzeigen nichts anzeigen, dann werden folgende Werte übergeben
			filter_ten  <= "1111111";
			filter_ones <= "1111111";
			db_ten  <= "1111111";
			db_ones <= "1111111";
			db_sign <= "1111111";
			db_letter_1 <= "1111111";
			db_letter_2 <= "1111111";
		end if;
	end process set_7seg;
	
	-- set_LCD -> Stellt die Anzeige am LCD-Display ein.
	set_LCD: process
	variable tmp : signed(7 downto 0);
	begin
		
		-- Warten auf steigende Flanke des 50 MHz Taktes
		wait until falling_edge(clk_50MHz);
		
		-- Ermitteln ob ein Filter oder die Lautstärke ausgewählt ist.
		if (filter > 0) then 
			-- Bei ausgewählten Filter, die Signal lcd_1, lcd_2 und lcd_3 so einstellen,
			-- dass diese die entsprechenden CGRAM-Werte für die entsprechenden Zeichen enthalten,
			-- welche in der zweiten Zeile an den ersten drei Stellen die Filterauswahl anzeigen.
			-- Die Anzeige ist nach folgendem Schema aufgebaut.
			-- Fxx
			-- Dabei ist xx ist Filternummer. Beim Filter 1 würde somit F01 im Display erscheinen.
			lcd_1 <= x"146";
			case filter is
				when x"1" => lcd_2 <= x"130"; lcd_3 <= x"131"; tmp := f1_db;
				when x"2" => lcd_2 <= x"130"; lcd_3 <= x"132"; tmp := f2_db;
				when x"3" => lcd_2 <= x"130"; lcd_3 <= x"133"; tmp := f3_db;
				when x"4" => lcd_2 <= x"130"; lcd_3 <= x"134"; tmp := f4_db;
				when x"5" => lcd_2 <= x"130"; lcd_3 <= x"135"; tmp := f5_db;
				when x"6" => lcd_2 <= x"130"; lcd_3 <= x"136"; tmp := f6_db;
				when x"7" => lcd_2 <= x"130"; lcd_3 <= x"137"; tmp := f7_db;
				when x"8" => lcd_2 <= x"130"; lcd_3 <= x"138"; tmp := f8_db;
				when x"9" => lcd_2 <= x"130"; lcd_3 <= x"139"; tmp := f9_db;
				when x"A" => lcd_2 <= x"131"; lcd_3 <= x"130"; tmp := f10_db;
				when x"B" => lcd_2 <= x"131"; lcd_3 <= x"131"; tmp := f11_db;
				when x"C" => lcd_2 <= x"131"; lcd_3 <= x"132"; tmp := f12_db;
				when x"D" => lcd_2 <= x"131"; lcd_3 <= x"133"; tmp := f13_db;
				when x"E" => lcd_2 <= x"131"; lcd_3 <= x"134"; tmp := f14_db;
				when others => null;
			end case;
		else
			-- Wenn kein Filter gewählt ist, dann ist zwangsläufig die Lautstärke ausgewählt wurden.
			-- Somit sind lcd_1, lcd_2 und lcd_3 auf die CGRAM-Werte für die Zeichen V, o und l einzustellen,
			-- damit im Display "Vol" erscheint für Volume.
			lcd_1 <= x"156";
			lcd_2 <= x"16F";
			lcd_3 <= x"16C";
			tmp := vol_db;
		end if;
		
		-- Die Signale lcd_4, lcd_5 und lcd_6 sind so einzustellen, dass diese
		-- auf die CGRAM-Werte für die entsprechenden Zeichen verweisen, damit
		-- im Display das Vorzeichen und der Verstärkungsfaktor angezeigt wird.
		case tmp is
			when x"14" => lcd_4 <= x"120"; lcd_5 <= x"132"; lcd_6 <= x"130";
			when x"0B" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"131";
			when x"0C" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"132";
			when x"0D" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"133";
			when x"0E" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"134";
			when x"0F" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"135";
			when x"10" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"136";
			when x"11" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"137";
			when x"12" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"138";
			when x"13" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"139";

			when x"0A" => lcd_4 <= x"120"; lcd_5 <= x"131"; lcd_6 <= x"130";
			when x"01" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"131";
			when x"02" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"132";
			when x"03" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"133";
			when x"04" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"134";
			when x"05" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"135";
			when x"06" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"136";
			when x"07" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"137";
			when x"08" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"138";
			when x"09" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"139";

			when x"00" => lcd_4 <= x"120"; lcd_5 <= x"130"; lcd_6 <= x"130";
			when x"FF" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"131";
			when x"FE" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"132";
			when x"FD" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"133";
			when x"FC" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"134";
			when x"FB" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"135";
			when x"FA" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"136";
			when x"F9" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"137";
			when x"F8" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"138";
			when x"F7" => lcd_4 <= x"12D"; lcd_5 <= x"130"; lcd_6 <= x"139";
			
			when x"F6" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"130";
			when x"F5" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"131";
			when x"F4" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"132";
			when x"F3" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"133";
			when x"F2" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"134";
			when x"F1" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"135";
			when x"F0" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"136";
			when x"EF" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"137";
			when x"EE" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"138";
			when x"ED" => lcd_4 <= x"12D"; lcd_5 <= x"131"; lcd_6 <= x"139";
			
			when x"EC" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"130";
			when x"EB" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"131";
			when x"EA" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"132";
			when x"E9" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"133";
			when x"E8" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"134";
			when x"E7" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"135";
			when x"E6" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"136";
			when x"E5" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"137";
			when x"E4" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"138";
			when x"E3" => lcd_4 <= x"12D"; lcd_5 <= x"132"; lcd_6 <= x"139";

			when x"E2" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"130";
			when x"E1" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"131";
			when x"E0" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"132";
			when x"DF" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"133";
			when x"DE" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"134";
			when x"DD" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"135";
			when x"DC" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"136";
			when x"DB" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"137";
			when x"DA" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"138";
			when x"D9" => lcd_4 <= x"12D"; lcd_5 <= x"133"; lcd_6 <= x"139";

			when x"D8" => lcd_4 <= x"12D"; lcd_5 <= x"134"; lcd_6 <= x"130";
			when others => null;
		end case;
	end process set_LCD;
	
	
	-- Alle berechneten bzw. ermittelten Werte werden den Ausgängen zugeführt.
	vol_value  <= vol_db;
	f1_value   <= f1_db;
	f2_value   <= f2_db;
	f3_value   <= f3_db;
	f4_value   <= f4_db;
	f5_value   <= f5_db;
	f6_value   <= f6_db;
	f7_value   <= f7_db;
	f8_value   <= f8_db;
	f9_value   <= f9_db;
	f10_value  <= f10_db;
	f11_value  <= f11_db;
	f12_value  <= f12_db;
	f13_value  <= f13_db;
	f14_value  <= f14_db;

end behavior;
